@@ -1,5 +1,4 @@
 #!/usr/bin/env python3
-# vim:fileencoding=utf-8
 # License: GPL v3 Copyright: 2018, Kovid Goyal <kovid at kovidgoyal.net>
 
 
@@ -11,6 +10,8 @@ from functools import partial
 from typing import TYPE_CHECKING, Any, Dict, FrozenSet, Generator, List, cast
 
 from kitty.types import run_once
+from kitty.utils import resolve_abs_or_config_path
+
 
 aliases = {'url_hints': 'hints'}
 if TYPE_CHECKING:
@@ -20,15 +21,15 @@ else:
 
 
 def resolved_kitten(k: str) -> str:
-    return aliases.get(k, k).replace('-', '_')
+    ans = aliases.get(k, k)
+    head, tail = os.path.split(ans)
+    tail = tail.replace('-', '_')
+    return os.path.join(head, tail)
 
 
 def path_to_custom_kitten(config_dir: str, kitten: str) -> str:
-    path = os.path.expanduser(kitten)
-    if not os.path.isabs(path):
-        path = os.path.join(config_dir, path)
-    path = os.path.abspath(path)
-    return path
+    path = resolve_abs_or_config_path(kitten, conf_dir=config_dir)
+    return os.path.abspath(path)
 
 
 @contextmanager
@@ -57,7 +58,7 @@ def import_kitten_main_module(config_dir: str, kitten: str) -> Dict[str, Any]:
         return {'start': g['main'], 'end': hr}
 
     kitten = resolved_kitten(kitten)
-    m = importlib.import_module('kittens.{}.main'.format(kitten))
+    m = importlib.import_module(f'kittens.{kitten}.main')
     return {'start': getattr(m, 'main'), 'end': getattr(m, 'handle_result', lambda *a, **k: None)}
 
 
@@ -84,14 +85,14 @@ def launch(args: List[str]) -> None:
     del args[:2]
     args = [kitten] + args
     os.environ['KITTY_CONFIG_DIRECTORY'] = config_dir
-    from kittens.tui.operations import clear_screen, reset_mode
+    from kittens.tui.operations import clear_screen, reset_mode, Mode
     set_debug(kitten)
     m = import_kitten_main_module(config_dir, kitten)
     try:
         result = m['start'](args)
     finally:
         sys.stdin = sys.__stdin__
-    print(reset_mode('ALTERNATE_SCREEN') + clear_screen(), end='')
+    print(reset_mode(Mode.ALTERNATE_SCREEN) + clear_screen(), end='')
     if result is not None:
         import json
         data = json.dumps(result)
@@ -107,7 +108,7 @@ def deserialize(output: str) -> Any:
             prefix, sz, rest = output.split(' ', 2)
             return json.loads(rest[:int(sz)])
         except Exception:
-            raise ValueError('Failed to parse kitten output: {!r}'.format(output))
+            raise ValueError(f'Failed to parse kitten output: {output!r}')
 
 
 def run_kitten(kitten: str, run_name: str = '__main__') -> None:
@@ -116,7 +117,7 @@ def run_kitten(kitten: str, run_name: str = '__main__') -> None:
     kitten = resolved_kitten(kitten)
     set_debug(kitten)
     try:
-        runpy.run_module('kittens.{}.main'.format(kitten), run_name=run_name)
+        runpy.run_module(f'kittens.{kitten}.main', run_name=run_name)
         return
     except ImportError:
         pass
@@ -129,19 +130,16 @@ def run_kitten(kitten: str, run_name: str = '__main__') -> None:
         print('Available builtin kittens:', file=sys.stderr)
         for kitten in all_kitten_names():
             print(kitten, file=sys.stderr)
-        raise SystemExit('No kitten named {}'.format(original_kitten_name))
+        raise SystemExit(f'No kitten named {original_kitten_name}')
     m = runpy.run_path(path, init_globals={'sys': sys, 'os': os}, run_name='__run_kitten__')
     m['main'](sys.argv)
 
 
 @run_once
 def all_kitten_names() -> FrozenSet[str]:
-    try:
-        from importlib.resources import contents
-    except ImportError:
-        from importlib_resources import contents  # type: ignore
+    from kitty.constants import list_kitty_resources
     ans = []
-    for name in contents('kittens'):
+    for name in list_kitty_resources('kittens'):
         if '__' not in name and '.' not in name and name != 'tui':
             ans.append(name)
     return frozenset(ans)
