@@ -39,7 +39,7 @@ from .fast_data_types import (
     set_application_quit_request, set_background_image, set_boss,
     set_clipboard_string, set_in_sequence_mode, set_options,
     set_os_window_size, set_os_window_title, thread_write, toggle_fullscreen,
-    toggle_maximized
+    toggle_maximized, toggle_secure_input
 )
 from .key_encoding import get_name_to_functional_number_map
 from .keys import get_shortcut, shortcut_matches
@@ -653,18 +653,13 @@ class Boss:
             msg = _('Are you sure you want to close this window?')
             if window.has_running_program:
                 msg += ' ' + _('It is running a program.')
-            self._run_kitten(
-                'ask', ['--type=yesno', '--message', msg],
-                window=window,
-                custom_callback=partial(self.handle_close_window_confirmation, window.id)
-            )
+            self.confirm(msg, self.handle_close_window_confirmation, window.id, window=window)
         else:
             self.mark_window_for_close(window)
 
-    def handle_close_window_confirmation(self, window_id: int, data: Dict[str, Any], *a: Any) -> None:
-        if data['response'] != 'y':
-            return
-        self.mark_window_for_close(window_id)
+    def handle_close_window_confirmation(self, allowed: bool, window_id: int) -> None:
+        if allowed:
+            self.mark_window_for_close(window_id)
 
     @ac('tab', 'Close the current tab')
     def close_tab(self, tab: Optional[Tab] = None) -> None:
@@ -672,13 +667,35 @@ class Boss:
         if tab:
             self.confirm_tab_close(tab)
 
-    def confirm(self, msg: str, callback: Callable[..., None], *args: Any,
-                window: Optional[Window] = None, confirm_on_cancel: bool = False) -> None:
+    def confirm(
+        self, msg: str,  # can contain newlines and ANSI formatting
+        callback: Callable[..., None],  # called with True or False and *args
+        *args: Any,  # passed to the callback function
+        window: Optional[Window] = None,  # the window associated with the confirmation
+        confirm_on_cancel: bool = False,  # on closing window
+        confirm_on_accept: bool = True,  # on pressing enter
+    ) -> None:
         def callback_(res: Dict[str, Any], x: int, boss: Boss) -> None:
             callback(res.get('response') == 'y', *args)
-        self._run_kitten('ask', ['--type=yesno', '--message', msg],
+        self._run_kitten('ask', ['--type=yesno', '--message', msg, '--default', 'y' if confirm_on_accept else 'n'],
                          window=window, custom_callback=callback_, default_data={'response': 'y' if confirm_on_cancel else 'n'})
 
+    def choose(
+        self, msg: str,  # can contain newlines and ANSI formatting
+        callback: Callable[..., None],  # called with the choice or empty string when aborted
+        *choices: str,   # The choices, see the help for the ask kitten for format of a choice
+        window: Optional[Window] = None,  # the window associated with the confirmation
+        default: str = '',  # the default choice when the user presses Enter
+    ) -> None:
+        def callback_(res: Dict[str, Any], x: int, boss: Boss) -> None:
+            callback(res.get('response') or '')
+        cmd = ['--type=choices', '--message', msg]
+        if default:
+            cmd += ['-d', default]
+        for c in choices:
+            cmd += ['-c', c]
+        self._run_kitten('ask', cmd, window=window, custom_callback=callback_, default_data={'response': ''})
+
     def confirm_tab_close(self, tab: Tab) -> None:
         x = get_options().confirm_os_window_close
         num = tab.number_of_windows_with_running_programs if x < 0 else len(tab)
@@ -721,6 +738,10 @@ class Boss:
     def toggle_maximized(self, os_window_id: int = 0) -> None:
         toggle_maximized(os_window_id)
 
+    @ac('misc', 'Toggle macOS secure keyboard entry')
+    def toggle_macos_secure_keyboard_entry(self) -> None:
+        toggle_secure_input()
+
     def start(self, first_os_window_id: int) -> None:
         if not getattr(self, 'io_thread_started', False):
             self.child_monitor.start()
@@ -1414,12 +1435,14 @@ class Boss:
     def set_tab_title(self) -> None:
         tab = self.active_tab
         if tab:
-            args = ['--name=tab-title', '--message', _('Enter the new title for this tab below.'), 'do_set_tab_title', str(tab.id)]
+            args = [
+                '--name=tab-title', '--message', _('Enter the new title for this tab below.'),
+                '--default', tab.name or tab.title, 'do_set_tab_title', str(tab.id)]
             self._run_kitten('ask', args)
 
     def do_set_tab_title(self, title: str, tab_id: int) -> None:
         tm = self.active_tab_manager
-        if tm is not None and title:
+        if tm is not None:
             tab_id = int(tab_id)
             for tab in tm.tabs:
                 if tab.id == tab_id:
@@ -2072,9 +2095,10 @@ class Boss:
 
     def choose_entry(
         self, title: str, entries: Iterable[Tuple[Union[_T, str, None], str]],
-        callback: Callable[[Union[_T, str, None]], None]
+        callback: Callable[[Union[_T, str, None]], None],
+        subtitle: str = ''
     ) -> Optional[Window]:
-        lines = [title, '']
+        lines = [title, subtitle, '']
         idx_map: List[Union[_T, str]] = []
         ans: Union[str, _T, None] = None
         fmt = ': {1}'
@@ -2110,8 +2134,19 @@ class Boss:
                     if tab.id == ans:
                         self.set_active_tab(tab)
 
+        def format_tab_title(tab: Tab) -> str:
+            w = 'windows' if tab.num_window_groups > 1 else 'window'
+            return f'{tab.title} [{tab.num_window_groups} {w}]'
+
         ct = self.active_tab
-        self.choose_entry('Choose a tab to switch to', ((t.id, t.title) for t in self.all_tabs if t is not ct), chosen)
+        st = ''
+        if ct is not None:
+            st = f'Current tab: {format_tab_title(ct)}'
+        self.choose_entry(
+            'Choose a tab to switch to',
+            ((t.id, format_tab_title(t)) for t in self.all_tabs if t is not ct),
+            chosen, subtitle=st
+        )
 
     @ac('win', '''
         Detach a window, moving it to another tab or OS Window
@@ -2200,19 +2235,31 @@ class Boss:
         if path == ":cocoa::application launched::":
             self.cocoa_application_launched = True
             return
-
-        def new_os_window() -> None:
-            self.new_os_window(path)
-
-        if self.cocoa_application_launched or not self.os_window_map:
-            return new_os_window()
+        from .open_actions import actions_for_launch
+        from .launch import force_window_launch
+        actions = list(actions_for_launch(path))
         tab = self.active_tab
-        if tab is None:
-            return new_os_window()
-        w = tab.active_window
-        self.new_window(path)
-        if w is not None:
-            tab.remove_window(w)
+        if tab is not None:
+            w = tab.active_window
+        else:
+            w = None
+        needs_window_replaced = not self.cocoa_application_launched or not self.os_window_map and w is not None and w.id == 1
+
+        def clear_initial_window() -> None:
+            if needs_window_replaced and tab is not None and w is not None:
+                tab.remove_window(w)
+
+        if not actions:
+            with force_window_launch(needs_window_replaced):
+                self.launch(kitty_exe(), '+runpy', f'print("The file:", {path!r}, "is of unknown type, cannot open it.");'
+                            'from kitty.utils import hold_till_enter; hold_till_enter(); raise SystemExit(1)')
+            clear_initial_window()
+        else:
+            with force_window_launch(needs_window_replaced):
+                self.dispatch_action(actions.pop(0))
+            clear_initial_window()
+            if actions:
+                self.drain_actions(actions)
 
     @ac('debug', 'Show the effective configuration kitty is running with')
     def debug_config(self) -> None:
