@@ -39,17 +39,16 @@ def generate_class(defn: Definition, loc: str) -> Tuple[str, str]:
     imports: Set[Tuple[str, str]] = set()
     tc_imports: Set[Tuple[str, str]] = set()
 
-    def type_name(x: type) -> str:
-        ans = x.__name__
-        if x.__module__ and x.__module__ != 'builtins':
-            imports.add((x.__module__, x.__name__))
-        return ans
-
     def option_type_as_str(x: Any) -> str:
-        if hasattr(x, '__name__'):
-            return type_name(x)
-        ans = repr(x)
-        ans = ans.replace('NoneType', 'None')
+        needs_import = False
+        if type(x) is type:
+            ans = x.__name__
+            needs_import = True
+        else:
+            ans = repr(x)
+            ans = ans.replace('NoneType', 'None')
+        if needs_import and getattr(x, '__module__', None) and x.__module__ not in ('builtins', 'typing'):
+            imports.add((x.__module__, x.__name__))
         return ans
 
     def option_type_data(option: Union[Option, MultiOption]) -> Tuple[Callable[[Any], Any], str]:
@@ -418,9 +417,9 @@ def generate_c_conversion(loc: str, ctypes: List[Option]) -> str:
 def write_output(loc: str, defn: Definition) -> None:
     cls, tc = generate_class(defn, loc)
     with open(os.path.join(*loc.split('.'), 'options', 'types.py'), 'w') as f:
-        f.write(cls + '\n')
+        f.write(f'{cls}\n')
     with open(os.path.join(*loc.split('.'), 'options', 'parse.py'), 'w') as f:
-        f.write(tc + '\n')
+        f.write(f'{tc}\n')
     ctypes = []
     for opt in defn.root_group.iter_all_non_groups():
         if isinstance(opt, Option) and opt.ctype:
@@ -428,7 +427,7 @@ def write_output(loc: str, defn: Definition) -> None:
     if ctypes:
         c = generate_c_conversion(loc, ctypes)
         with open(os.path.join(*loc.split('.'), 'options', 'to-c-generated.h'), 'w') as f:
-            f.write(c + '\n')
+            f.write(f'{c}\n')
 
 
 def main() -> None:
@@ -454,6 +453,6 @@ def main() -> None:
     loc = package_name
     cls, tc = generate_class(defn, loc)
     with open(os.path.join(os.path.dirname(path), 'kitten_options_types.py'), 'w') as f:
-        f.write(cls + '\n')
+        f.write(f'{cls}\n')
     with open(os.path.join(os.path.dirname(path), 'kitten_options_parse.py'), 'w') as f:
-        f.write(tc + '\n')
+        f.write(f'{tc}\n')
