@@ -226,6 +226,7 @@ extern bool init_mouse(PyObject *module);
 extern bool init_kittens(PyObject *module);
 extern bool init_logging(PyObject *module);
 extern bool init_png_reader(PyObject *module);
+extern bool init_utmp(PyObject *module);
 #ifdef __APPLE__
 extern int init_CoreText(PyObject *);
 extern bool init_cocoa(PyObject *module);
@@ -235,10 +236,26 @@ extern bool init_freetype_library(PyObject*);
 extern bool init_freetype_render_ui_text(PyObject*);
 #endif
 
+static unsigned
+shift_to_first_set_bit(CellAttrs x) {
+    unsigned num_of_bits = 8 * sizeof(x.val);
+    unsigned ans = 0;
+    while (num_of_bits--) {
+        if (x.val & 1) return ans;
+        x.val >>= 1;
+        ans++;
+    }
+    return ans;
+}
+
 
 EXPORTED PyMODINIT_FUNC
 PyInit_fast_data_types(void) {
     PyObject *m;
+    if (sizeof(CellAttrs) != 2u) {
+        PyErr_SetString(PyExc_RuntimeError, "Size of CellAttrs is not 2 on this platform");
+        return NULL;
+    }
 
     m = PyModule_Create(&module);
     if (m == NULL) return NULL;
@@ -277,14 +294,13 @@ PyInit_fast_data_types(void) {
     if (!init_freetype_render_ui_text(m)) return NULL;
 #endif
     if (!init_fonts(m)) return NULL;
+    if (!init_utmp(m)) return NULL;
 
-    PyModule_AddIntConstant(m, "BOLD", BOLD_SHIFT);
-    PyModule_AddIntConstant(m, "ITALIC", ITALIC_SHIFT);
-    PyModule_AddIntConstant(m, "REVERSE", REVERSE_SHIFT);
-    PyModule_AddIntConstant(m, "STRIKETHROUGH", STRIKE_SHIFT);
-    PyModule_AddIntConstant(m, "DIM", DIM_SHIFT);
-    PyModule_AddIntConstant(m, "DECORATION", DECORATION_SHIFT);
-    PyModule_AddIntConstant(m, "MARK", MARK_SHIFT);
+    CellAttrs a;
+#define s(name, attr) { a.val = 0; a.attr = 1; PyModule_AddIntConstant(m, #name, shift_to_first_set_bit(a)); }
+    s(BOLD, bold); s(ITALIC, italic); s(REVERSE, reverse); s(MARK, mark);
+    s(STRIKETHROUGH, strike); s(DIM, dim); s(DECORATION, decoration);
+#undef s
     PyModule_AddIntConstant(m, "MARK_MASK", MARK_MASK);
     PyModule_AddStringMacro(m, ERROR_PREFIX);
 #ifdef KITTY_VCS_REV
@@ -302,6 +318,7 @@ PyInit_fast_data_types(void) {
     PyModule_AddIntMacro(m, DCS);
     PyModule_AddIntMacro(m, APC);
     PyModule_AddIntMacro(m, OSC);
+    PyModule_AddIntMacro(m, FILE_TRANSFER_CODE);
 
     return m;
 }
