@@ -7,14 +7,11 @@
 
 #define EXTRA_INIT if (PyModule_AddFunctions(module, module_methods) != 0) return false;
 #define MAX_KEY_SIZE 256u
-#if __linux__
-#define HAS_SENDFILE
-#endif
-
 #include "disk-cache.h"
 #include "safe-wrappers.h"
 #include "kitty-uthash.h"
 #include "loop-utils.h"
+#include "fast-file-copy.h"
 #include "threading.h"
 #include "cross-platform-random.h"
 #include <structmember.h>
@@ -22,9 +19,6 @@
 #include <sys/stat.h>
 #include <fcntl.h>
 #include <time.h>
-#ifdef HAS_SENDFILE
-#include <sys/sendfile.h>
-#endif
 
 
 typedef struct {
@@ -113,50 +107,6 @@ open_cache_file(const char *cache_path) {
 }
 
 // Write loop {{{
-static bool
-copy_between_files(int infd, int outfd, off_t in_pos, size_t len, uint8_t *buf, size_t bufsz) {
-#ifdef HAS_SENDFILE
-    (void)buf; (void)bufsz;
-    while (len) {
-        off_t r = in_pos;
-        ssize_t n = sendfile(outfd, infd, &r, len);
-        if (n < 0) {
-            if (errno != EAGAIN) return false;
-            continue;
-        }
-        in_pos += n; len -= n;
-    }
-#else
-    while (len) {
-        ssize_t amt_read = pread(infd, buf, MIN(len, bufsz), in_pos);
-        if (amt_read < 0) {
-            if (errno == EINTR || errno == EAGAIN) continue;
-            return false;
-        }
-        if (amt_read == 0) {
-            errno = EIO;
-            return false;
-        }
-        len -= amt_read;
-        in_pos += amt_read;
-        uint8_t *p = buf;
-        while(amt_read) {
-            ssize_t amt_written = write(outfd, p, amt_read);
-            if (amt_written < 0) {
-                if (errno == EINTR || errno == EAGAIN) continue;
-                return false;
-            }
-            if (amt_written == 0) {
-                errno = EIO;
-                return false;
-            }
-            amt_read -= amt_written;
-            p += amt_written;
-        }
-    }
-#endif
-    return true;
-}
 
 static off_t
 size_of_cache_file(DiskCache *self) {
@@ -183,8 +133,7 @@ static void
 defrag(DiskCache *self) {
     int new_cache_file = -1;
     FREE_AFTER_FUNCTION DefragEntry *defrag_entries = NULL;
-    FREE_AFTER_FUNCTION uint8_t *buf = NULL;
-    const size_t bufsz = 1024 * 1024;
+    AutoFreeFastFileCopyBuffer fcb = {0};
     bool lock_released = false, ok = false;
 
     off_t size_on_disk = size_of_cache_file(self);
@@ -215,17 +164,13 @@ defrag(DiskCache *self) {
         goto cleanup;
     }
     lseek(new_cache_file, 0, SEEK_SET);
-#ifndef HAS_SENDFILE
-    buf = malloc(bufsz);
-    if (!buf) goto cleanup;
-#endif
 
     mutex(unlock); lock_released = true;
 
     off_t current_pos = 0;
     for (size_t i = 0; i < num_entries_to_defrag; i++) {
         DefragEntry *e = defrag_entries + i;
-        if (!copy_between_files(self->cache_file_fd, new_cache_file, e->old_offset, e->data_sz, buf, bufsz)) {
+        if (!copy_between_files(self->cache_file_fd, new_cache_file, e->old_offset, e->data_sz, &fcb)) {
             perror("Failed to copy data to new disk cache file during defrag");
             goto cleanup;
         }
