@@ -24,6 +24,7 @@ extern void cocoa_set_activation_policy(bool);
 extern void cocoa_set_titlebar_appearance(void *w, unsigned int theme);
 extern void cocoa_set_titlebar_color(void *w, color_type color);
 extern bool cocoa_alt_option_key_pressed(unsigned long);
+extern void cocoa_toggle_secure_keyboard_entry(void);
 extern size_t cocoa_get_workspace_ids(void *w, size_t *workspace_ids, size_t array_sz);
 extern monotonic_t cocoa_cursor_blink_interval(void);
 
@@ -1181,6 +1182,14 @@ glfw_window_hint(PyObject UNUSED *self, PyObject *args) {
 
 // }}}
 
+static PyObject*
+toggle_secure_input(PYNOARG) {
+#ifdef __APPLE__
+    cocoa_toggle_secure_keyboard_entry();
+#endif
+    Py_RETURN_NONE;
+}
+
 static PyObject*
 get_clipboard_string(PYNOARG) {
     OSWindow *w = current_os_window();
@@ -1555,6 +1564,11 @@ strip_csi(PyObject *self UNUSED, PyObject *src) {
     return PyUnicode_FromString(buf);
 }
 
+void
+set_ignore_os_keyboard_processing(bool enabled) {
+    glfwSetIgnoreOSKeyboardProcessing(enabled);
+}
+
 // Boilerplate {{{
 
 static PyMethodDef module_methods[] = {
@@ -1562,6 +1576,7 @@ static PyMethodDef module_methods[] = {
     {"create_os_window", (PyCFunction)(void (*) (void))(create_os_window), METH_VARARGS | METH_KEYWORDS, NULL},
     METHODB(set_default_window_icon, METH_VARARGS),
     METHODB(get_clipboard_string, METH_NOARGS),
+    METHODB(toggle_secure_input, METH_NOARGS),
     METHODB(get_content_scale_for_window, METH_NOARGS),
     METHODB(ring_bell, METH_NOARGS),
     METHODB(set_clipboard_string, METH_VARARGS),
