@@ -8,6 +8,7 @@ from .boss import Boss
 from .child import Child
 from .cli import parse_args
 from .cli_stub import LaunchCLIOptions
+from .constants import kitty_exe, shell_path
 from .fast_data_types import patch_color_profiles, set_clipboard_string
 from .options.utils import env as parse_env
 from .tabs import Tab
@@ -71,6 +72,11 @@ Using :code:`name=` will set to empty string and just :code:`name` will
 remove the environment variable.
 
 
+--hold
+type=bool-set
+Keep the window open even after the command being executed exits.
+
+
 --copy-colors
 type=bool-set
 Set the colors of the newly created window to be the same as the colors in the
@@ -300,6 +306,28 @@ def apply_colors(window: Window, spec: Sequence[str]) -> None:
     patch_color_profiles(colors, profiles, True)
 
 
+class ForceWindowLaunch:
+
+    def __init__(self) -> None:
+        self.force = False
+
+    def __bool__(self) -> bool:
+        return self.force
+
+    def __call__(self, force: bool) -> 'ForceWindowLaunch':
+        self.force = force
+        return self
+
+    def __enter__(self) -> None:
+        pass
+
+    def __exit__(self, *a: object) -> None:
+        self.force = False
+
+
+force_window_launch = ForceWindowLaunch()
+
+
 def launch(
     boss: Boss,
     opts: LaunchCLIOptions,
@@ -341,7 +369,7 @@ def launch(
         if opts.stdin_add_formatting:
             if q in ('@screen', '@screen_scrollback', '@alternate', '@alternate_scrollback',
                      '@first_cmd_output_on_screen', '@last_cmd_output', '@last_visited_cmd_output'):
-                q = '@ansi_' + q[1:]
+                q = f'@ansi_{q[1:]}'
         if opts.stdin_add_line_wrap_markers:
             q += '_wrap'
         penv, stdin = boss.process_stdin_source(window=active, stdin=q, copy_pipe_data=pipe_data)
@@ -383,6 +411,8 @@ def launch(
         if exe:
             final_cmd[0] = exe
         kw['cmd'] = final_cmd
+    if force_window_launch and opts.type not in ('background', 'clipboard', 'primary'):
+        opts.type = 'window'
     if opts.type == 'overlay' and active:
         kw['overlay_for'] = active.id
     if opts.type == 'background':
@@ -398,6 +428,9 @@ def launch(
             else:
                 set_primary_selection(stdin)
     else:
+        if opts.hold:
+            cmd = kw['cmd'] or [shell_path]
+            kw['cmd'] = [kitty_exe(), '+hold'] + cmd
         if force_target_tab:
             tab = target_tab
         else:
