@@ -327,23 +327,31 @@ detect_url(Screen *screen, unsigned int x, unsigned int y) {
     else set_mouse_cursor_for_screen(screen);
 }
 
+static bool
+set_mouse_position(Window *w, bool *mouse_cell_changed, bool *cell_half_changed) {
+    unsigned int x = 0, y = 0;
+    bool in_left_half_of_cell = false;
+    if (!cell_for_pos(w, &x, &y, &in_left_half_of_cell, global_state.callback_os_window)) return false;
+    *mouse_cell_changed = x != w->mouse_pos.cell_x || y != w->mouse_pos.cell_y;
+    *cell_half_changed = in_left_half_of_cell != w->mouse_pos.in_left_half_of_cell;
+    w->mouse_pos.cell_x = x; w->mouse_pos.cell_y = y;
+    w->mouse_pos.in_left_half_of_cell = in_left_half_of_cell;
+    return true;
+}
+
 HANDLER(handle_move_event) {
     modifiers &= ~GLFW_LOCK_MASK;
-    unsigned int x = 0, y = 0;
     if (OPT(focus_follows_mouse)) {
         Tab *t = global_state.callback_os_window->tabs + global_state.callback_os_window->active_tab;
         if (window_idx != t->active_window) {
             call_boss(switch_focus_to, "K", t->windows[window_idx].id);
         }
     }
-    bool in_left_half_of_cell = false;
-    if (!cell_for_pos(w, &x, &y, &in_left_half_of_cell, global_state.callback_os_window)) return;
+    bool mouse_cell_changed = false;
+    bool cell_half_changed = false;
+    if (!set_mouse_position(w, &mouse_cell_changed, &cell_half_changed)) return;
     Screen *screen = w->render_data.screen;
-    if(OPT(detect_urls)) detect_url(screen, x, y);
-    bool mouse_cell_changed = x != w->mouse_pos.cell_x || y != w->mouse_pos.cell_y;
-    bool cell_half_changed = in_left_half_of_cell != w->mouse_pos.in_left_half_of_cell;
-    w->mouse_pos.cell_x = x; w->mouse_pos.cell_y = y;
-    w->mouse_pos.in_left_half_of_cell = in_left_half_of_cell;
+    if(OPT(detect_urls)) detect_url(screen, w->mouse_pos.cell_x, w->mouse_pos.cell_y);
     bool in_tracking_mode = (
         screen->modes.mouse_tracking_mode == ANY_MODE ||
         (screen->modes.mouse_tracking_mode == MOTION_MODE && button >= 0));
@@ -401,7 +409,7 @@ multi_click_count(Window *w, int button) {
 
 static void
 add_press(Window *w, int button, int modifiers) {
-    if (button < 0 || button > (ssize_t)arraysz(w->click_queues)) return;
+    if (button < 0 || button >= (ssize_t)arraysz(w->click_queues)) return;
     modifiers &= ~GLFW_LOCK_MASK;
     ClickQueue *q = &w->click_queues[button];
     if (q->length == CLICK_QUEUE_SZ) { memmove(q->clicks, q->clicks + 1, sizeof(Click) * (CLICK_QUEUE_SZ - 1)); q->length--; }
@@ -497,14 +505,22 @@ HANDLER(handle_button_event) {
     }
     Screen *screen = w->render_data.screen;
     if (!screen) return;
+    bool a, b;
+    if (!set_mouse_position(w, &a, &b)) return;
+    id_type wid = w->id;
     if (!dispatch_mouse_event(w, button, is_release ? -1 : 1, modifiers, screen->modes.mouse_tracking_mode != 0)) {
         if (screen->modes.mouse_tracking_mode != 0) {
             int sz = encode_mouse_button(w, button, is_release ? RELEASE : PRESS, modifiers);
             if (sz > 0) { mouse_event_buf[sz] = 0; write_escape_code_to_child(screen, CSI, mouse_event_buf); }
         }
     }
-    if (is_release) dispatch_possible_click(w, button, modifiers);
-    else add_press(w, button, modifiers);
+    // the windows array might have been re-alloced in dispatch_mouse_event
+    w = NULL;
+    for (size_t i = 0; i < t->num_windows && !w; i++) if (t->windows[i].id == wid) w = t->windows + i;
+    if (w) {
+        if (is_release) dispatch_possible_click(w, button, modifiers);
+        else add_press(w, button, modifiers);
+    }
 }
 
 static int
