@@ -42,6 +42,8 @@ character_key_name_aliases_with_ascii_lowercase: Dict[str, str] = character_key_
 for x in 'ABCDEFGHIJKLMNOPQRSTUVWXYZ':
     character_key_name_aliases_with_ascii_lowercase[x] = x.lower()
 sequence_sep = '>'
+mouse_button_map = {'left': 'b1', 'middle': 'b3', 'right': 'b2'}
+mouse_trigger_count_map = {'doubleclick': -3, 'click': -2, 'release': -1, 'press': 1, 'doublepress': 2, 'triplepress': 3}
 FuncArgsType = Tuple[str, Sequence[Any]]
 func_with_args = KeyFuncWrapper[FuncArgsType]()
 DELETE_ENV_VAR = '_delete_this_env_var_'
@@ -377,7 +379,7 @@ def parse_mods(parts: Iterable[str], sc: str) -> Optional[int]:
     mods = 0
     for m in parts:
         try:
-            mods |= getattr(defines, 'GLFW_MOD_' + map_mod(m.upper()))
+            mods |= getattr(defines, f'GLFW_MOD_{map_mod(m.upper())}')
         except AttributeError:
             if m.upper() != 'NONE':
                 log_error(f'Shortcut: {sc} has unknown modifier, ignoring')
@@ -392,7 +394,7 @@ def to_modifiers(val: str) -> int:
 
 def parse_shortcut(sc: str) -> SingleKey:
     if sc.endswith('+') and len(sc) > 1:
-        sc = sc[:-1] + 'plus'
+        sc = f'{sc[:-1]}plus'
     parts = sc.split('+')
     mods = 0
     if len(parts) > 1:
@@ -498,15 +500,14 @@ def scrollback_pager_history_size(x: str) -> int:
     return min(ans, 4096 * 1024 * 1024 - 1)
 
 
+# "single" for backwards compat
+url_style_map = {'none': 0, 'single': 1, 'straight': 1, 'double': 2, 'curly': 3, 'dotted': 4, 'dashed': 5}
+
+
 def url_style(x: str) -> int:
     return url_style_map.get(x, url_style_map['curly'])
 
 
-url_style_map = {
-    v: i for i, v in enumerate('none single double curly'.split())
-}
-
-
 def url_prefixes(x: str) -> Tuple[str, ...]:
     return tuple(a.lower() for a in x.replace(',', ' ').split())
 
@@ -638,10 +639,19 @@ def tab_fade(x: str) -> Tuple[float, ...]:
     return tuple(map(unit_float, x.split()))
 
 
-def tab_activity_symbol(x: str) -> Optional[str]:
+def tab_activity_symbol(x: str) -> str:
     if x == 'none':
-        return None
-    return tab_title_template(x) or None
+        return ''
+    return tab_title_template(x)
+
+
+def bell_on_tab(x: str) -> str:
+    xl = x.lower()
+    if xl in ('yes', 'y', 'true'):
+        return '🔔 '
+    if xl in ('no', 'n', 'false', 'none'):
+        return ''
+    return tab_title_template(x)
 
 
 def tab_title_template(x: str) -> str:
@@ -894,13 +904,13 @@ def resolve_aliases_and_parse_actions(
             parts = rest.split(maxsplit=1)
             if parts[0] != alias.name:
                 continue
-            new_defn = possible_alias + ' ' + alias.value + ((' ' + parts[1]) if len(parts) > 1 else '')
+            new_defn = f'{possible_alias} {alias.value}{f" {parts[1]}" if len(parts) > 1 else ""}'
             new_aliases = aliases.copy()
             new_aliases[possible_alias] = [a for a in aliases[possible_alias] if a is not alias]
             yield from resolve_aliases_and_parse_actions(new_defn, new_aliases, map_type)
             return
         else:  # action_alias
-            new_defn = alias.value + ((' ' + rest) if rest else '')
+            new_defn = f'{alias.value} {rest}' if rest else alias.value
             new_aliases = aliases.copy()
             new_aliases.pop(possible_alias)
             yield from resolve_aliases_and_parse_actions(new_defn, new_aliases, map_type)
@@ -908,7 +918,7 @@ def resolve_aliases_and_parse_actions(
 
     if possible_alias == 'combine':
         sep, rest = rest.split(maxsplit=1)
-        parts = re.split(r'\s*' + re.escape(sep) + r'\s*', rest)
+        parts = re.split(fr'\s*{re.escape(sep)}\s*', rest)
         for x in parts:
             if x:
                 yield from resolve_aliases_and_parse_actions(x, aliases, map_type)
@@ -1054,13 +1064,13 @@ def parse_mouse_map(val: str) -> Iterable[MouseMapping]:
         obutton = parts[0].lower()
         mods = 0
     try:
-        b = {'left': 'b1', 'middle': 'b3', 'right': 'b2'}.get(obutton, obutton)[1:]
+        b = mouse_button_map.get(obutton, obutton)[1:]
         button = getattr(defines, f'GLFW_MOUSE_BUTTON_{b}')
     except Exception:
         log_error(f'Mouse button: {xbutton} not recognized, ignoring')
         return
     try:
-        count = {'doubleclick': -3, 'click': -2, 'release': -1, 'press': 1, 'doublepress': 2, 'triplepress': 3}[event.lower()]
+        count = mouse_trigger_count_map[event.lower()]
     except KeyError:
         log_error(f'Mouse event type: {event} not recognized, ignoring')
         return
@@ -1103,8 +1113,7 @@ def deprecated_send_text(key: str, val: str, ans: Dict[str, Any]) -> None:
     parts = val.split(' ')
 
     def abort(msg: str) -> None:
-        log_error('Send text: {} is invalid ({}), ignoring'.format(
-            val, msg))
+        log_error(f'Send text: {val} is invalid ({msg}), ignoring')
 
     if len(parts) < 3:
         return abort('Incomplete')
