@@ -3,6 +3,7 @@
 
 import os
 from functools import lru_cache, partial, wraps
+from string import Formatter as StringFormatter
 from typing import (
     Any, Callable, Dict, List, NamedTuple, Optional, Sequence, Tuple, Union
 )
@@ -17,7 +18,7 @@ from .fast_data_types import (
 from .rgb import alpha_blend, color_as_sgr, color_from_int, to_color
 from .types import WindowGeometry, run_once
 from .typing import EdgeLiteral, PowerlineStyle
-from .utils import color_as_int, log_error
+from .utils import color_as_int, log_error, sgr_sanitizer_pat
 
 
 class TabBarData(NamedTuple):
@@ -38,8 +39,7 @@ class DrawData(NamedTuple):
     leading_spaces: int
     sep: str
     trailing_spaces: int
-    bell_on_tab: bool
-    bell_fg: int
+    bell_on_tab: str
     alpha: Sequence[float]
     active_fg: Color
     active_bg: Color
@@ -48,7 +48,7 @@ class DrawData(NamedTuple):
     default_bg: Color
     title_template: str
     active_title_template: Optional[str]
-    tab_activity_symbol: Optional[str]
+    tab_activity_symbol: str
     powerline_style: PowerlineStyle
     tab_bar_edge: EdgeLiteral
 
@@ -102,14 +102,14 @@ class ColorFormatter:
             ans = '9'
         elif q == 'tab':
             col = color_from_int((self.draw_data.tab_bg if self.which == '4' else self.draw_data.tab_fg)(self.tab_data))
-            ans = '8' + color_as_sgr(col)
+            ans = f'8{color_as_sgr(col)}'
         else:
             if name.startswith('_'):
-                q = '#' + name[1:]
+                q = f'#{name[1:]}'
             c = to_color(q)
             if c is None:
                 raise AttributeError(f'{name} is not a valid color')
-            ans = '8' + color_as_sgr(c)
+            ans = f'8{color_as_sgr(c)}'
         return f'\x1b[{self.which}{ans}m'
 
 
@@ -152,48 +152,9 @@ class ExtraData:
     next_tab: Optional[TabBarData] = None
 
 
-def draw_title(draw_data: DrawData, screen: Screen, tab: TabBarData, index: int) -> None:
-    if tab.needs_attention and draw_data.bell_on_tab:
-        fg = screen.cursor.fg
-        screen.cursor.fg = draw_data.bell_fg
-        screen.draw('🔔 ')
-        screen.cursor.fg = fg
-    if tab.has_activity_since_last_focus and draw_data.tab_activity_symbol:
-        fg = screen.cursor.fg
-        screen.cursor.fg = draw_data.bell_fg
-        screen.draw(draw_data.tab_activity_symbol)
-        screen.cursor.fg = fg
-
-    template = draw_data.title_template
-    if tab.is_active and draw_data.active_title_template is not None:
-        template = draw_data.active_title_template
-    try:
-        data = {
-            'index': index,
-            'layout_name': tab.layout_name,
-            'num_windows': tab.num_windows,
-            'num_window_groups': tab.num_window_groups,
-            'title': tab.title,
-        }
-        ColorFormatter.draw_data = draw_data
-        ColorFormatter.tab_data = tab
-        eval_locals = {
-            'index': index,
-            'layout_name': tab.layout_name,
-            'num_windows': tab.num_windows,
-            'num_window_groups': tab.num_window_groups,
-            'title': tab.title,
-            'fmt': Formatter,
-            'sup': SupSub(data),
-            'sub': SupSub(data, True),
-        }
-        title = eval(compile_template(template), {'__builtins__': {}}, eval_locals)
-    except Exception as e:
-        report_template_failure(template, str(e))
-        title = tab.title
+def draw_attributed_string(title: str, screen: Screen) -> None:
     if '\x1b' in title:
-        import re
-        for x in re.split('(\x1b\\[[^m]*m)', title):
+        for x in sgr_sanitizer_pat(for_splitting=True).split(title):
             if x.startswith('\x1b') and x.endswith('m'):
                 screen.apply_sgr(x[2:-1])
             else:
@@ -202,6 +163,55 @@ def draw_title(draw_data: DrawData, screen: Screen, tab: TabBarData, index: int)
         screen.draw(title)
 
 
+@lru_cache(maxsize=16)
+def template_has_field(template: str, field: str) -> bool:
+    q = StringFormatter()
+    for (literal_text, field_name, format_spec, conversion) in q.parse(template):
+        if field_name and field in field_name.split():
+            return True
+    return False
+
+
+def draw_title(draw_data: DrawData, screen: Screen, tab: TabBarData, index: int) -> None:
+    data = {
+        'index': index,
+        'layout_name': tab.layout_name,
+        'num_windows': tab.num_windows,
+        'num_window_groups': tab.num_window_groups,
+        'title': tab.title,
+    }
+    ColorFormatter.draw_data = draw_data
+    ColorFormatter.tab_data = tab
+    eval_locals = {
+        'index': index,
+        'layout_name': tab.layout_name,
+        'num_windows': tab.num_windows,
+        'num_window_groups': tab.num_window_groups,
+        'title': tab.title,
+        'fmt': Formatter,
+        'sup': SupSub(data),
+        'sub': SupSub(data, True),
+        'bell_symbol': draw_data.bell_on_tab if tab.needs_attention else '',
+        'activity_symbol': draw_data.tab_activity_symbol if tab.has_activity_since_last_focus else '',
+    }
+    template = draw_data.title_template
+    if tab.is_active and draw_data.active_title_template is not None:
+        template = draw_data.active_title_template
+    prefix = ''
+    if eval_locals['bell_symbol'] and not template_has_field(template, 'bell_symbol'):
+        prefix = '{bell_symbol}'
+    if eval_locals['activity_symbol'] and not template_has_field(template, 'activity_symbol'):
+        prefix += '{activity_symbol}'
+    if prefix:
+        template = '{fmt.fg.red}' + prefix + '{fmt.fg.default}' + template
+    try:
+        title = eval(compile_template(template), {'__builtins__': {}}, eval_locals)
+    except Exception as e:
+        report_template_failure(template, str(e))
+        title = tab.title
+    draw_attributed_string(title, screen)
+
+
 DrawTabFunc = Callable[[DrawData, Screen, TabBarData, int, int, int, bool, ExtraData], int]
 
 
@@ -435,9 +445,8 @@ class TabBar:
 
         self.active_bg = as_rgb(color_as_int(opts.active_tab_background))
         self.active_fg = as_rgb(color_as_int(opts.active_tab_foreground))
-        self.bell_fg = as_rgb(0xff0000)
         self.draw_data = DrawData(
-            self.leading_spaces, self.sep, self.trailing_spaces, opts.bell_on_tab, self.bell_fg,
+            self.leading_spaces, self.sep, self.trailing_spaces, opts.bell_on_tab,
             opts.tab_fade, opts.active_tab_foreground, opts.active_tab_background,
             opts.inactive_tab_foreground, opts.inactive_tab_background,
             opts.tab_bar_background or opts.background, opts.tab_title_template,
@@ -557,7 +566,7 @@ class TabBar:
             if s.cursor.x > s.columns - max_title_length and t is not last_tab:
                 s.cursor.x = s.columns - 2
                 s.cursor.bg = as_rgb(color_as_int(self.draw_data.default_bg))
-                s.cursor.fg = self.bell_fg
+                s.cursor.fg = as_rgb(0xff0000)
                 s.draw(' …')
                 break
         s.erase_in_line(0, False)  # Ensure no long titles bleed after the last tab
