@@ -1,5 +1,4 @@
 #!/usr/bin/env python3
-# vim:fileencoding=utf-8
 # License: GPL v3 Copyright: 2016, Kovid Goyal <kovid at kovidgoyal.net>
 
 import os
@@ -7,37 +6,50 @@ import stat
 import weakref
 from collections import deque
 from contextlib import suppress
-from functools import partial
 from operator import attrgetter
+from time import monotonic
 from typing import (
-    Any, Deque, Dict, Generator, Iterator, List, NamedTuple, Optional, Pattern,
-    Sequence, Tuple, Union, cast
+    Any, Deque, Dict, Generator, Iterable, Iterator, List, NamedTuple,
+    Optional, Pattern, Sequence, Set, Tuple, Union, cast
 )
 
-from .borders import Borders
+from .borders import Border, Borders
 from .child import Child
 from .cli_stub import CLIOptions
 from .constants import appname, kitty_exe
 from .fast_data_types import (
-    add_tab, attach_window, detach_window, get_boss, get_options,
-    mark_tab_bar_dirty, next_window_id, remove_tab, remove_window, ring_bell,
-    set_active_tab, set_active_window, swap_tabs, sync_os_window_title
+    GLFW_MOUSE_BUTTON_LEFT, GLFW_MOUSE_BUTTON_MIDDLE, GLFW_PRESS, GLFW_RELEASE,
+    add_tab, attach_window, detach_window, get_boss, get_click_interval,
+    get_options, mark_tab_bar_dirty, next_window_id, remove_tab, remove_window,
+    ring_bell, set_active_tab, set_active_window, swap_tabs,
+    sync_os_window_title
 )
-from .layout.base import Layout, Rect
+from .layout.base import Layout
 from .layout.interface import create_layout_object_for, evict_cached_layouts
 from .tab_bar import TabBar, TabBarData
+from .types import ac
 from .typing import EdgeLiteral, SessionTab, SessionType, TypedDict
 from .utils import log_error, platform_window_id, resolved_shell
 from .window import Watchers, Window, WindowDict
 from .window_list import WindowList
 
 
+class TabMouseEvent(NamedTuple):
+    button: int
+    modifiers: int
+    action: int
+    at: float
+    tab_idx: Optional[int]
+
+
 class TabDict(TypedDict):
     id: int
     is_focused: bool
     title: str
     layout: str
     layout_state: Dict[str, Any]
+    layout_opts: Dict[str, Any]
+    enabled_layouts: List[str]
     windows: List[WindowDict]
     active_window_history: List[int]
 
@@ -76,6 +88,11 @@ def add_active_id_to_history(items: Deque[int], item_id: int, maxlen: int = 64)
 
 class Tab:  # {{{
 
+    active_fg: Optional[int] = None
+    active_bg: Optional[int] = None
+    inactive_fg: Optional[int] = None
+    inactive_bg: Optional[int] = None
+
     def __init__(
         self,
         tab_manager: 'TabManager',
@@ -88,14 +105,12 @@ class Tab:  # {{{
         self.os_window_id: int = tab_manager.os_window_id
         self.id: int = add_tab(self.os_window_id)
         if not self.id:
-            raise Exception('No OS window with id {} found, or tab counter has wrapped'.format(self.os_window_id))
+            raise Exception(f'No OS window with id {self.os_window_id} found, or tab counter has wrapped')
         self.args = tab_manager.args
         self.name = getattr(session_tab, 'name', '')
         self.enabled_layouts = [x.lower() for x in getattr(session_tab, 'enabled_layouts', None) or get_options().enabled_layouts]
         self.borders = Borders(self.os_window_id, self.id)
         self.windows = WindowList(self)
-        for i, which in enumerate('first second third fourth fifth sixth seventh eighth ninth tenth'.split()):
-            setattr(self, which + '_window', partial(self.nth_window, num=i))
         self._last_used_layout: Optional[str] = None
         self._current_layout_name: Optional[str] = None
         self.cwd = self.args.directory
@@ -115,14 +130,17 @@ class Tab:  # {{{
             self._set_current_layout(l0)
             self.startup(session_tab)
 
-    def apply_options(self) -> None:
-        for window in self:
-            window.apply_options()
-        self.enabled_layouts = [x.lower() for x in get_options().enabled_layouts] or ['tall']
+    def set_enabled_layouts(self, val: Iterable[str]) -> None:
+        self.enabled_layouts = [x.lower() for x in val] or ['tall']
         if self.current_layout.name not in self.enabled_layouts:
             self._set_current_layout(self.enabled_layouts[0])
         self.relayout()
 
+    def apply_options(self) -> None:
+        for window in self:
+            window.apply_options()
+        self.set_enabled_layouts(get_options().enabled_layouts)
+
     def take_over_from(self, other_tab: 'Tab') -> None:
         self.name, self.cwd = other_tab.name, other_tab.cwd
         self.enabled_layouts = list(other_tab.enabled_layouts)
@@ -137,6 +155,7 @@ class Tab:  # {{{
         for window in self.windows:
             window.change_tab(self)
             attach_window(self.os_window_id, self.id, window.id)
+        self.active_window_changed()
         self.relayout()
 
     def _set_current_layout(self, layout_name: str) -> None:
@@ -161,6 +180,9 @@ class Tab:  # {{{
             'window_list': self.windows.serialize_state(),
             'current_layout': self._current_layout_name,
             'last_used_layout': self._last_used_layout,
+            'layout_opts': self.current_layout.layout_opts,
+            'layout_state': self.current_layout.layout_state,
+            'enabled_layouts': self.enabled_layouts,
             'name': self.name,
         }
 
@@ -188,6 +210,14 @@ class Tab:  # {{{
     def title(self) -> str:
         return cast(str, getattr(self.active_window, 'title', appname))
 
+    @property
+    def number_of_windows_with_running_programs(self) -> int:
+        ans = 0
+        for window in self:
+            if window.has_running_program:
+                ans += 1
+        return ans
+
     def set_title(self, title: str) -> None:
         self.name = title or ''
         self.mark_tab_bar_dirty()
@@ -213,7 +243,7 @@ class Tab:  # {{{
             ly = self.current_layout
             self.borders(
                 all_windows=self.windows,
-                current_layout=ly, extra_blank_rects=tm.blank_rects,
+                current_layout=ly, tab_bar_rects=tm.tab_bar_rects,
                 draw_window_borders=(ly.needs_window_borders and self.windows.num_visble_groups > 1) or ly.must_draw_borders
             )
             if w is not None:
@@ -222,8 +252,8 @@ class Tab:  # {{{
     def create_layout_object(self, name: str) -> Layout:
         return create_layout_object_for(name, self.os_window_id, self.id)
 
+    @ac('lay', 'Go to the next enabled layout')
     def next_layout(self) -> None:
-        '@ac:lay: Go to the next enabled layout'
         if len(self.enabled_layouts) > 1:
             for i, layout_name in enumerate(self.enabled_layouts):
                 if layout_name == self.current_layout.full_name:
@@ -235,39 +265,39 @@ class Tab:  # {{{
             self._set_current_layout(nl)
             self.relayout()
 
+    @ac('lay', 'Go to the previously used layout')
     def last_used_layout(self) -> None:
-        '@ac:lay: Go to the previously used layout'
         if len(self.enabled_layouts) > 1 and self._last_used_layout and self._last_used_layout != self._current_layout_name:
             self._set_current_layout(self._last_used_layout)
             self.relayout()
 
-    def goto_layout(self, layout_name: str, raise_exception: bool = False) -> None:
-        '''
-        @ac:lay: Switch to the named layout
+    @ac('lay', '''
+        Switch to the named layout
 
         For example::
 
             map f1 goto_layout tall
-        '''
+        ''')
+    def goto_layout(self, layout_name: str, raise_exception: bool = False) -> None:
         layout_name = layout_name.lower()
         if layout_name not in self.enabled_layouts:
             if raise_exception:
                 raise ValueError(layout_name)
-            log_error('Unknown or disabled layout: {}'.format(layout_name))
+            log_error(f'Unknown or disabled layout: {layout_name}')
             return
         self._set_current_layout(layout_name)
         self.relayout()
 
-    def toggle_layout(self, layout_name: str) -> None:
-        '''
-        @ac:lay: Toggle the named layout
+    @ac('lay', '''
+        Toggle the named layout
 
         Switches to the named layout if another layout is current, otherwise
         switches to the last used layout. Useful to "zoom" a window temporarily
         by switching to the stack layout. For example::
 
             map f1 toggle_layout stack
-        '''
+        ''')
+    def toggle_layout(self, layout_name: str) -> None:
         if self._current_layout_name == layout_name:
             self.last_used_layout()
         else:
@@ -280,12 +310,12 @@ class Tab:  # {{{
             return None
         return 'Could not resize'
 
-    def resize_window(self, quality: str, increment: int) -> None:
-        '''
-        @ac:win: Resize the active window by the specified amount
+    @ac('win', '''
+        Resize the active window by the specified amount
 
         See :ref:`window_resizing` for details.
-        '''
+        ''')
+    def resize_window(self, quality: str, increment: int) -> None:
         if increment < 1:
             raise ValueError(increment)
         is_horizontal = quality in ('wider', 'narrower')
@@ -296,13 +326,13 @@ class Tab:  # {{{
             if get_options().enable_audio_bell:
                 ring_bell()
 
+    @ac('win', 'Reset window sizes undoing any dynamic resizing of windows')
     def reset_window_sizes(self) -> None:
-        '@ac:win:Reset window sizes undoing any dynamic resizing of windows'
         if self.current_layout.remove_all_biases():
             self.relayout()
 
+    @ac('lay', 'Perform a layout specific action. See :doc:`layouts` for details')
     def layout_action(self, action_name: str, args: Sequence[str]) -> None:
-        '@ac:lay: Perform a layout specific action. See :doc:`layouts` for details'
         ret = self.current_layout.layout_action(action_name, args, self.windows)
         if ret is None:
             ring_bell()
@@ -330,12 +360,11 @@ class Tab:  # {{{
                 else:
                     cmd = resolved_shell(get_options())
                     check_for_suitability = False
-                cmd = self.args.args or resolved_shell(get_options())
         if check_for_suitability:
             old_exe = cmd[0]
             if not os.path.isabs(old_exe):
-                import shutil
-                actual_exe = shutil.which(old_exe)
+                from .utils import which
+                actual_exe = which(old_exe)
                 old_exe = actual_exe if actual_exe else os.path.abspath(old_exe)
             try:
                 is_executable = os.access(old_exe, os.X_OK)
@@ -424,14 +453,8 @@ class Tab:  # {{{
             allow_remote_control=allow_remote_control, watchers=special_window.watchers
         )
 
-    def close_window(self) -> None:
-        '@ac:win: Close the currently active window'
-        w = self.active_window
-        if w is not None:
-            self.remove_window(w)
-
+    @ac('win', 'Close all windows in the tab other than the currently active window')
     def close_other_windows_in_tab(self) -> None:
-        '@ac:win: Close all windows in the tab other than the currently active window'
         if len(self.windows) > 1:
             active_window = self.active_window
             for window in tuple(self.windows):
@@ -468,33 +491,76 @@ class Tab:  # {{{
     def get_nth_window(self, n: int) -> Optional[Window]:
         if self.windows:
             return self.current_layout.nth_window(self.windows, n)
+        return None
 
-    def nth_window(self, num: int = 0) -> None:
-        '''
-        @ac:win: Focus the nth window if positive or the previously active windows if negative
-
-        For example, to ficus the previously active window::
+    @ac('win', '''
+        Focus the nth window if positive or the previously active windows if negative. When the number is larger
+        than the number of windows focus the last window. For example::
 
+            # focus the previously active window
             map ctrl+p nth_window -1
-        '''
+            # focus the first window
+            map ctrl+1 nth_window 0
+        ''')
+    def nth_window(self, num: int = 0) -> None:
         if self.windows:
             if num < 0:
                 self.windows.make_previous_group_active(-num)
-            else:
-                self.current_layout.activate_nth_window(self.windows, num)
+            elif self.windows.num_groups:
+                self.current_layout.activate_nth_window(self.windows, min(num, self.windows.num_groups - 1))
             self.relayout_borders()
 
+    @ac('win', 'Focus the first window')
+    def first_window(self) -> None:
+        self.nth_window(0)
+
+    @ac('win', 'Focus the second window')
+    def second_window(self) -> None:
+        self.nth_window(1)
+
+    @ac('win', 'Focus the third window')
+    def third_window(self) -> None:
+        self.nth_window(2)
+
+    @ac('win', 'Focus the fourth window')
+    def fourth_window(self) -> None:
+        self.nth_window(3)
+
+    @ac('win', 'Focus the fifth window')
+    def fifth_window(self) -> None:
+        self.nth_window(4)
+
+    @ac('win', 'Focus the sixth window')
+    def sixth_window(self) -> None:
+        self.nth_window(5)
+
+    @ac('win', 'Focus the seventh window')
+    def seventh_window(self) -> None:
+        self.nth_window(6)
+
+    @ac('win', 'Focus the eighth window')
+    def eighth_window(self) -> None:
+        self.nth_window(7)
+
+    @ac('win', 'Focus the ninth window')
+    def ninth_window(self) -> None:
+        self.nth_window(8)
+
+    @ac('win', 'Focus the tenth window')
+    def tenth_window(self) -> None:
+        self.nth_window(9)
+
     def _next_window(self, delta: int = 1) -> None:
         if len(self.windows) > 1:
             self.current_layout.next_window(self.windows, delta)
             self.relayout_borders()
 
+    @ac('win', 'Focus the next window in the current tab')
     def next_window(self) -> None:
-        '@ac:win: Focus the next window in the current tab'
         self._next_window()
 
+    @ac('win', 'Focus the previous window in the current tab')
     def previous_window(self) -> None:
-        '@ac:win: Focus the previous window in the current tab'
         self._next_window(-1)
 
     prev_window = previous_window
@@ -509,35 +575,43 @@ class Tab:  # {{{
 
         if groups:
             return groups[0]
+        return None
+
+    def nth_active_window_id(self, n: int = 0) -> int:
+        if n <= 0:
+            return self.active_window.id if self.active_window else 0
+        ids = tuple(reversed(self.windows.active_window_history))
+        return ids[min(n - 1, len(ids) - 1)] if ids else 0
 
     def neighboring_group_id(self, which: EdgeLiteral) -> Optional[int]:
         neighbors = self.current_layout.neighbors(self.windows)
         candidates = neighbors.get(which)
         if candidates:
             return self.most_recent_group(candidates)
+        return None
 
-    def neighboring_window(self, which: EdgeLiteral) -> None:
-        '''
-        @ac:win: Focus the neighboring window in the current tab
+    @ac('win', '''
+        Focus the neighboring window in the current tab
 
         For example::
 
             map ctrl+left neighboring_window left
             map ctrl+down neighboring_window bottom
-        '''
+        ''')
+    def neighboring_window(self, which: EdgeLiteral) -> None:
         neighbor = self.neighboring_group_id(which)
         if neighbor:
             self.windows.set_active_group(neighbor)
 
-    def move_window(self, delta: Union[EdgeLiteral, int] = 1) -> None:
-        '''
-        @ac:win: Move the window in the specified direction
+    @ac('win', '''
+        Move the window in the specified direction
 
         For example::
 
             map ctrl+left move_window left
             map ctrl+down move_window bottom
-        '''
+        ''')
+    def move_window(self, delta: Union[EdgeLiteral, int] = 1) -> None:
         if isinstance(delta, int):
             if self.current_layout.move_window(self.windows, delta):
                 self.relayout()
@@ -547,25 +621,59 @@ class Tab:  # {{{
                 if self.current_layout.move_window_to_group(self.windows, neighbor):
                     self.relayout()
 
+    def swap_active_window_with(self, window_id: int) -> None:
+        group = self.windows.group_for_window(window_id)
+        if group is not None:
+            w = self.active_window
+            if w is not None and w.id != window_id:
+                if self.current_layout.move_window_to_group(self.windows, group.id):
+                    self.relayout()
+
+    @property
+    def all_window_ids_except_active_window(self) -> Set[int]:
+        all_window_ids = {w.id for w in self}
+        aw = self.active_window
+        if aw is not None:
+            all_window_ids.discard(aw.id)
+        return all_window_ids
+
+    @ac('win', '''
+        Focus a visible window by pressing the number of the window. Window numbers are displayed
+        over the windows for easy selection in this mode. See :opt:`visual_window_select_characters`.
+        ''')
+    def focus_visible_window(self) -> None:
+        def callback(tab: Optional[Tab], window: Optional[Window]) -> None:
+            if tab and window:
+                tab.set_active_window(window)
+
+        get_boss().visual_window_select_action(self, callback, 'Choose window to switch to', only_window_ids=self.all_window_ids_except_active_window)
+
+    @ac('win', 'Swap the current window with another window in the current tab, selected visually. See :opt:`visual_window_select_characters`')
+    def swap_with_window(self) -> None:
+        def callback(tab: Optional[Tab], window: Optional[Window]) -> None:
+            if tab and window:
+                tab.swap_active_window_with(window.id)
+        get_boss().visual_window_select_action(self, callback, 'Choose window to swap with', only_window_ids=self.all_window_ids_except_active_window)
+
+    @ac('win', 'Move active window to the top (make it the first window)')
     def move_window_to_top(self) -> None:
-        '@ac:win: Move active window to the top (make it the first window)'
         n = self.windows.active_group_idx
         if n > 0:
             self.move_window(-n)
 
+    @ac('win', 'Move active window forward (swap it with the next window)')
     def move_window_forward(self) -> None:
-        '@ac:win: Move active window forward (swap it with the next window)'
         self.move_window()
 
+    @ac('win', 'Move active window backward (swap it with the previous window)')
     def move_window_backward(self) -> None:
-        '@ac:win: Move active window backward (swap it with the previous window)'
         self.move_window(-1)
 
     def list_windows(self, active_window: Optional[Window], self_window: Optional[Window] = None) -> Generator[WindowDict, None, None]:
         for w in self:
             yield w.as_dict(is_focused=w is active_window, is_self=w is self_window)
 
-    def matches(self, field: str, pat: Pattern) -> bool:
+    def matches(self, field: str, pat: 'Pattern[str]') -> bool:
         if field == 'id':
             return bool(pat.pattern == str(self.id))
         if field == 'title':
@@ -592,7 +700,7 @@ class Tab:  # {{{
         self.windows = WindowList(self)
 
     def __repr__(self) -> str:
-        return 'Tab(title={}, id={})'.format(self.name or self.title, hex(id(self)))
+        return f'Tab(title={self.name or self.title}, id={hex(id(self))})'
 
     def make_active(self) -> None:
         tm = self.tab_manager_ref()
@@ -606,6 +714,7 @@ class TabManager:  # {{{
     def __init__(self, os_window_id: int, args: CLIOptions, wm_class: str, wm_name: str, startup_session: Optional[SessionType] = None):
         self.os_window_id = os_window_id
         self.wm_class = wm_class
+        self.recent_mouse_events: Deque[TabMouseEvent] = deque()
         self.wm_name = wm_name
         self.last_active_tab_id = None
         self.args = args
@@ -731,6 +840,7 @@ class TabManager:  # {{{
             delta = -1 if loc == 'left' else 1
             idx = (len(self.tabs) + self.active_tab_idx + delta) % len(self.tabs)
             return self.tabs[idx]
+        return None
 
     def goto_tab(self, tab_num: int) -> None:
         if tab_num >= len(self.tabs):
@@ -747,6 +857,12 @@ class TabManager:  # {{{
                     self.set_active_tab_idx(idx)
                     break
 
+    def nth_active_tab(self, n: int = 0) -> Optional[Tab]:
+        if n <= 0:
+            return self.active_tab
+        tab_ids = tuple(reversed(self.active_tab_history))
+        return self.tab_for_id(tab_ids[min(n - 1, len(tab_ids) - 1)]) if tab_ids else None
+
     def __iter__(self) -> Iterator[Tab]:
         return iter(self.tabs)
 
@@ -761,6 +877,8 @@ class TabManager:  # {{{
                 'title': tab.name or tab.title,
                 'layout': str(tab.current_layout.name),
                 'layout_state': tab.current_layout.layout_state(),
+                'layout_opts': tab.current_layout.layout_opts.serialized(),
+                'enabled_layouts': tab.enabled_layouts,
                 'windows': list(tab.list_windows(active_window, self_window)),
                 'active_window_history': list(tab.windows.active_window_history),
             }
@@ -785,19 +903,27 @@ class TabManager:  # {{{
         t = self.active_tab
         if t is not None:
             return t.active_window
+        return None
+
+    @property
+    def number_of_windows_with_running_programs(self) -> int:
+        count = 0
+        for tab in self:
+            count += tab.number_of_windows_with_running_programs
+        return count
 
     @property
     def number_of_windows(self) -> int:
         count = 0
         for tab in self:
-            for window in tab:
-                count += 1
+            count += len(tab)
         return count
 
     def tab_for_id(self, tab_id: int) -> Optional[Tab]:
         for t in self.tabs:
             if t.id == tab_id:
                 return t
+        return None
 
     def move_tab(self, delta: int = 1) -> None:
         if len(self.tabs) > 1:
@@ -895,20 +1021,41 @@ class TabManager:  # {{{
             ans.append(TabBarData(
                 title, t is at, needs_attention,
                 len(t), t.num_window_groups, t.current_layout.name or '',
-                has_activity_since_last_focus
+                has_activity_since_last_focus, t.active_fg, t.active_bg,
+                t.inactive_fg, t.inactive_bg
             ))
         return ans
 
-    def activate_tab_at(self, x: int, is_double: bool = False) -> None:
+    def handle_click_on_tab(self, x: int, button: int, modifiers: int, action: int) -> None:
         i = self.tab_bar.tab_at(x)
+        now = monotonic()
         if i is None:
-            if is_double:
-                self.new_tab()
+            if button == GLFW_MOUSE_BUTTON_LEFT and action == GLFW_RELEASE and len(self.recent_mouse_events) > 2:
+                ci = get_click_interval()
+                prev, prev2 = self.recent_mouse_events[-1], self.recent_mouse_events[-2]
+                if (
+                    prev.button == button and prev2.button == button and
+                    prev.action == GLFW_PRESS and prev2.action == GLFW_RELEASE and
+                    prev.tab_idx is None and prev2.tab_idx is None and
+                    now - prev.at <= ci and now - prev2.at <= 2 * ci
+                ):  # double click
+                    self.new_tab()
+                    self.recent_mouse_events.clear()
+                    return
         else:
-            self.set_active_tab_idx(i)
+            if action == GLFW_PRESS and button == GLFW_MOUSE_BUTTON_LEFT:
+                self.set_active_tab_idx(i)
+            elif button == GLFW_MOUSE_BUTTON_MIDDLE and action == GLFW_RELEASE and self.recent_mouse_events:
+                p = self.recent_mouse_events[-1]
+                if p.button == button and p.action == GLFW_PRESS and p.tab_idx == i:
+                    tab = self.tabs[i]
+                    get_boss().close_tab(tab)
+        self.recent_mouse_events.append(TabMouseEvent(button, modifiers, action, now, i))
+        if len(self.recent_mouse_events) > 5:
+            self.recent_mouse_events.popleft()
 
     @property
-    def blank_rects(self) -> Tuple[Rect, ...]:
+    def tab_bar_rects(self) -> Tuple[Border, ...]:
         return self.tab_bar.blank_rects if self.tab_bar_should_be_visible else ()
 
     def destroy(self) -> None:
