@@ -1,17 +1,15 @@
 #!/usr/bin/env python3
-# vim:fileencoding=utf-8
 # License: GPL v3 Copyright: 2016, Kovid Goyal <kovid at kovidgoyal.net>
 
 import os
 from unittest import TestCase
 
-from kitty.config import (
-    Options, defaults, finalize_keys, finalize_mouse_mappings
-)
+from kitty.config import finalize_keys, finalize_mouse_mappings
 from kitty.fast_data_types import (
     Cursor, HistoryBuf, LineBuf, Screen, set_options
 )
 from kitty.options.parse import merge_result_dicts
+from kitty.options.types import Options, defaults
 from kitty.types import MouseEvent
 
 
@@ -50,23 +48,32 @@ class Callbacks:
     def open_url(self, url: str, hyperlink_id: int) -> None:
         self.open_urls.append((url, hyperlink_id))
 
+    def clipboard_control(self, data: str, is_partial: bool = False) -> None:
+        self.cc_buf.append((data, is_partial))
+
     def clear(self) -> None:
         self.wtcbuf = b''
         self.iconbuf = self.titlebuf = self.colorbuf = self.ctbuf = ''
         self.iutf8 = True
         self.notifications = []
         self.open_urls = []
+        self.cc_buf = []
+        self.bell_count = 0
+
+    def on_bell(self) -> None:
+        self.bell_count += 1
 
     def on_activity_since_last_focus(self) -> None:
         pass
 
     def on_mouse_event(self, event):
         ev = MouseEvent(**event)
-        action = self.opts.mousemap.get(ev)
-        if action is None:
+        action_def = self.opts.mousemap.get(ev)
+        if not action_def:
             return False
         self.current_mouse_button = ev.button
-        getattr(self, action.func)(*action.args)
+        for action in self.opts.alias_map.resolve_aliases(action_def, 'mouse_map'):
+            getattr(self, action.func)(*action.args)
         self.current_mouse_button = 0
         return True
 
@@ -75,7 +82,7 @@ def filled_line_buf(ynum=5, xnum=5, cursor=Cursor()):
     ans = LineBuf(ynum, xnum)
     cursor.x = 0
     for i in range(ynum):
-        t = ('{}'.format(i)) * xnum
+        t = (f'{i}') * xnum
         ans.line(i).set_text(t, 0, xnum, cursor)
     return ans
 
@@ -100,7 +107,7 @@ def filled_history_buf(ynum=5, xnum=5, cursor=Cursor()):
 class BaseTest(TestCase):
 
     ae = TestCase.assertEqual
-    maxDiff = 2000
+    maxDiff = 2048
     is_ci = os.environ.get('CI') == 'true'
 
     def set_options(self, options=None):
@@ -108,8 +115,8 @@ class BaseTest(TestCase):
         if options:
             final_options.update(options)
         options = Options(merge_result_dicts(defaults._asdict(), final_options))
-        finalize_keys(options)
-        finalize_mouse_mappings(options)
+        finalize_keys(options, {})
+        finalize_mouse_mappings(options, {})
         set_options(options)
         return options
 
